// $Id$
/**
 * Copyright (C) 2009 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.definedterm;

import org.apache.commons.lang.StringUtils;
import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.StyledString.Styler;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.TextStyle;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.TermBase;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author l.morris
 * @date 9 Dec 2011
 *
 */
public class TermLabelProvider extends StyledCellLabelProvider {

    private static Color vocColor = Display.getCurrent().getSystemColor(SWT.COLOR_BLUE);
    private Styler vocStyler;

    public TermLabelProvider() {
    }

    public TermLabelProvider(Styler vocStyler){
        this.vocStyler = vocStyler;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.eclipse.jface.viewers.StyledCellLabelProvider#update(org.eclipse.
     * jface.viewers.ViewerCell)
     */
    @Override
    public void update(ViewerCell cell) {
        Object element = cell.getElement();
        int columnIndex = cell.getColumnIndex();

        String text = getText(element);
        cell.setText(text);

        if (element instanceof TermVocabulary) {
            StyledString styledString = new StyledString(text, getVocabularyStyler());
            StyleRange[] styleRanges;
            styleRanges = styledString.getStyleRanges();
            cell.setStyleRanges(styleRanges);
        }
        super.update(cell);
    }

    public StyledString getStyledText(Object element) {

        if (element instanceof TermVocabulary) {
            new StyledString(getText(element), getVocabularyStyler());
        }
        return new StyledString(getText(element), StyledString.QUALIFIER_STYLER);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
     */

    public String getText(Object element) {

        if (element instanceof DefinedTermBase) {
            DefinedTermBase<?> dtb = (DefinedTermBase<?>) element;
            if (StringUtils.isNotBlank(dtb.getIdInVocabulary())) {
                return dtb.getIdInVocabulary() + " : " + dtb.getLabel(PreferencesUtil.getGlobalLanguage());
            } else {
                return dtb.getLabel(PreferencesUtil.getGlobalLanguage());
            }
        } else if (element instanceof TermBase) {
            return ((TermBase) element).getTitleCache();
        }

        // FIXME : must throw an exception here
        return element.toString();
    }

    private Styler getVocabularyStyler() {
        if (vocStyler == null) {
            vocStyler = new Styler() {
                @Override
                public void applyStyles(TextStyle textStyle) {
                    textStyle.foreground = vocColor;
                }
            };
        }
        return vocStyler;
    }

}
