package eu.etaxonomy.taxeditor.handler.defaultHandler;

import java.util.UUID;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.ParameterType;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.taxeditor.model.MessagingUtils;

public abstract class DefaultOpenHandlerBase <T> extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        String commandId = event.getCommand().getId();
        String uuidParameterId = commandId+".uuid";
        //check if uuid parameter is set
        if(event.getParameter(uuidParameterId)!=null){
            Object object = event.getObjectParameterForExecution(uuidParameterId);
            ParameterType parameterType;
            try {
                parameterType = event.getCommand().getParameterType(uuidParameterId);
                if(parameterType.isCompatible(object)){
                    T entity = getEntity((UUID) object);
                    open(event, entity);
                }
            } catch (NotDefinedException e) {
                MessagingUtils.error(DefaultOpenHandlerBase.class, "Error while opening classification!", e);
            }
        }
        //if not try current selection
        else{
            ISelection selection = HandlerUtil.getCurrentSelection(event);
            if(selection instanceof IStructuredSelection){
                open(event, (T) ((IStructuredSelection) selection).getFirstElement());
            }
        }
        return null;
    }


    protected abstract T getEntity(UUID uuid);

    protected abstract void open(ExecutionEvent event, T entity);

}