// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;

/**
 * @author k.luther
 * @date Oct 5, 2015
 *
 */
public class DeleteResultMessagingUtils extends MessagingUtils {
    /**
     * Displays a message {@link org.eclipse.jface.dialogs.MessageDialog}.
     *
     * @param title
     * @param source
     * @param message
     */
    public static void messageDialogWithDetails(DeleteResult result, String message, String pluginId) {
        if (result.isOk() && result.getExceptions().isEmpty()){
            return;
        }
        List<String> details = new ArrayList<String>();
        String title= "";

        if (result.getExceptions().size() > 0){
            for (Exception e:result.getExceptions()){
                details.add(e.getMessage());
            }
            if (result.isOk()){
                title = "Delete was successfull.";
            } else {
                title = "Delete was aborted.";
            }
        }
        StringBuffer relatedObjectsString = new StringBuffer();
        List<String> relatedObjects = new ArrayList<String>();
        if (result.getUpdatedObjects().size() > 0){
            Iterator<CdmBase> objects = result.getRelatedObjects().iterator();
            while (objects.hasNext()){
                CdmBase object = objects.next();
                if (object instanceof IdentifiableEntity){

                    relatedObjects.add(((IdentifiableEntity)object).getTitleCache() );
                } else{
                    relatedObjects.add(object.getUserFriendlyTypeName());
                }

            }

            for (String relatedObject: relatedObjects){

                relatedObjectsString.append(relatedObject);
                relatedObjectsString.append(System.lineSeparator());
            }
        }
        message = message + "\n" ;
        String stackTraceWithContext = getContextInfo(details);
        CdmDeleteErrorDialog ced = new CdmDeleteErrorDialog(AbstractUtility.getShell(), title, message,new Status(IStatus.INFO, pluginId, relatedObjectsString.toString(), result.getExceptions().iterator().next()), stackTraceWithContext, result.getUpdatedObjects().toArray());

        ced.open();



    }
}
