/**
 * 
 */
package eu.etaxonomy.taxeditor.model;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.IPartService;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.PlatformUI;

/**
 * <p>TaxeditorPartService class.</p>
 *
 * @author n.hoffmann
 * @version $Id: $
 */
public class TaxeditorPartService implements IPartListener2{

	/** Constant <code>PART_ACTIVATED=10</code> */
	public static final int PART_ACTIVATED = 10;
	/** Constant <code>PART_BROUGHT_TO_TOP=20</code> */
	public static final int PART_BROUGHT_TO_TOP = 20;
	/** Constant <code>PART_CLOSED=30</code> */
	public static final int PART_CLOSED = 30;
	/** Constant <code>PART_DEAVTICATED=40</code> */
	public static final int PART_DEAVTICATED = 40;
	/** Constant <code>PART_HIDDEN=50</code> */
	public static final int PART_HIDDEN = 50;
	/** Constant <code>PART_INPUT_CHANGED=60</code> */
	public static final int PART_INPUT_CHANGED = 60;
	/** Constant <code>PART_OPENED=70</code> */
	public static final int PART_OPENED = 70;
	/** Constant <code>PART_VISIBLE=80</code> */
	public static final int PART_VISIBLE = 80;
	
	/** Constant <code>instance</code> */
	public static TaxeditorPartService instance = new TaxeditorPartService();
	
	private IPartService partService;
	
	private Map<Integer, Set<IPartChangeListener>> listenerMap = new HashMap<Integer, Set<IPartChangeListener>>();
	
	private TaxeditorPartService() {
		partService = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getPartService();
		partService.addPartListener(this);
	}
	
	/**
	 * <p>Getter for the field <code>instance</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.model.TaxeditorPartService} object.
	 */
	public static TaxeditorPartService getInstance(){
		return instance;
	}
	
	/**
	 * <p>addListener</p>
	 *
	 * @param eventType a {@link java.lang.Integer} object.
	 * @param listener a {@link eu.etaxonomy.taxeditor.model.IPartChangeListener} object.
	 */
	public void addListener(Integer eventType, IPartChangeListener listener){
		Set<IPartChangeListener> partChangeListeners = listenerMap.get(eventType);
		
		if(partChangeListeners == null){
			partChangeListeners = new HashSet<IPartChangeListener>();
			listenerMap.put(eventType, partChangeListeners);
		}
		
		partChangeListeners.add(listener);
	}
	
	/**
	 * <p>removeListener</p>
	 *
	 * @param eventType a {@link java.lang.Integer} object.
	 * @param listener a {@link eu.etaxonomy.taxeditor.model.IPartChangeListener} object.
	 */
	public void removeListener(Integer eventType, IPartChangeListener listener){
		Set<IPartChangeListener> partChangeListeners = listenerMap.get(eventType);
		
		if(partChangeListeners == null){
			return;
		}
		
		partChangeListeners.remove(listener);
	}
	
	private void notifyListeners(Integer eventType, IWorkbenchPartReference partRef){
		Set<IPartChangeListener> partChangeListeners = listenerMap.get(eventType);
		
		if(partChangeListeners == null){
			return;
		}
		
		for(IPartChangeListener listener : partChangeListeners){
			listener.partChanged(eventType, partRef);
		}
	}
	

	/** {@inheritDoc} */
	public void partActivated(IWorkbenchPartReference partRef) {
		notifyListeners(PART_ACTIVATED, partRef);
	}

	/** {@inheritDoc} */
	public void partBroughtToTop(IWorkbenchPartReference partRef) {
		notifyListeners(PART_BROUGHT_TO_TOP, partRef);
	}

	/** {@inheritDoc} */
	public void partClosed(IWorkbenchPartReference partRef) {
		notifyListeners(PART_CLOSED, partRef);
	}

	/** {@inheritDoc} */
	public void partDeactivated(IWorkbenchPartReference partRef) {
		notifyListeners(PART_DEAVTICATED, partRef);
	}

	/** {@inheritDoc} */
	public void partOpened(IWorkbenchPartReference partRef) {
		notifyListeners(PART_OPENED, partRef);
	}

	/** {@inheritDoc} */
	public void partHidden(IWorkbenchPartReference partRef) {
		notifyListeners(PART_HIDDEN, partRef);
	}

	/** {@inheritDoc} */
	public void partVisible(IWorkbenchPartReference partRef) {
		notifyListeners(PART_VISIBLE, partRef);
	}

	/** {@inheritDoc} */
	public void partInputChanged(IWorkbenchPartReference partRef) {
		notifyListeners(PART_INPUT_CHANGED, partRef);
	}
}
