// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.newWizard;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbench;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;

/**
 * <p>Abstract AbstractNewEntityWizard class.</p>
 *
 * @author n.hoffmann
 * @created Jun 1, 2010
 * @version 1.0
 */
public abstract class AbstractNewEntityWizard<T> extends Wizard implements
		INewWizard, IConversationEnabled {

	private ConversationHolder conversation;

	protected CdmFormFactory formFactory;

	private T entity;

	private IWorkbench workbench;

	private IStructuredSelection selection;

	/**
	 * <p>Constructor for AbstractNewEntityWizard.</p>
	 *
	 * @param <T> a T object.
	 */
	public AbstractNewEntityWizard(){
		setWindowTitle(String.format("New %s", getEntityName()));
	}

	/**
	 * FIXME there might be a smarter way to do this,
	 *
	 * @return
	 */
	protected abstract String getEntityName();

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	/** {@inheritDoc} */
	@Override
	public boolean performFinish() {
		saveEntity();

		conversation.commit();
		conversation.close();
		return true;
	}

	/**
	 * <p>Getter for the field <code>entity</code>.</p>
	 *
	 * @return a T object.
	 */
	public T getEntity() {
		return entity;
	}

	/**
	 * <p>Setter for the field <code>entity</code>.</p>
	 *
	 * @param entity a T object.
	 */
	public void setEntity(T entity){
		this.entity = entity;
	}

	/**
	 * Adds the entity to the current persistence context
	 */
	protected abstract void saveEntity();

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	/** {@inheritDoc} */
	@Override
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.workbench = workbench != null ? workbench : AbstractUtility.getWorkbench();

		if(selection == null){
			ISelectionService service = this.workbench.getActiveWorkbenchWindow().getSelectionService();
			if(service.getSelection() instanceof IStructuredSelection){
				selection = (IStructuredSelection) service.getSelection();
			}
		}
		this.selection = selection;

		formFactory = new CdmFormFactory(Display.getCurrent(), null);
	 	conversation = CdmStore.createConversation();
	 	entity = createNewEntity();

	}

	/**
	 * <p>createNewEntity</p>
	 *
	 * @return a T object.
	 */
	protected abstract T createNewEntity();

	/**
	 * <p>getConversationHolder</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	@Override
    public ConversationHolder getConversationHolder() {
		return conversation;
	}

	/** {@inheritDoc} */
	@Override
    public void update(CdmDataChangeMap changeEvents) {}

	/**
	 * @return the workbench
	 */
	public IWorkbench getWorkbench() {
		return workbench;
	}

	/**
	 * @return the selection
	 */
	public IStructuredSelection getSelection() {
		return selection;
	}
}
