// $Id$
/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.IDescribable;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.media.Media;

/**
 * @author pplitzner
 * @date 04.12.2013
 *
 */
public abstract class AbstractDescriptionPostOperation<T extends IDescribable<D>, D extends DescriptionBase<?>> extends AbstractPostOperation<T> {

    protected D description;

    private final boolean isImageGallery;

    /**
     *
     */
    public AbstractDescriptionPostOperation(String label, IUndoContext undoContext,
            T describable, IPostOperationEnabled postOperationEnabled) {
        this(label, undoContext, describable, postOperationEnabled, false);
    }

    /**
     *
     */
    public AbstractDescriptionPostOperation(String label, IUndoContext undoContext,
            T describable, IPostOperationEnabled postOperationEnabled,  boolean isImageGallery) {
        super(label, undoContext, describable, postOperationEnabled);
        this.isImageGallery = isImageGallery;
    }

    /* (non-Javadoc)
     * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
     */
    /** {@inheritDoc} */
    @Override
    public IStatus execute(IProgressMonitor monitor, IAdaptable info)
            throws ExecutionException {

        initDescription();
        monitor.worked(20);

        if(isImageGallery){
            description.setImageGallery(isImageGallery);
            // add the description element to hold the media elements for this image gallery
            TextData element = TextData.NewInstance(Feature.IMAGE());
            element.addMedia(Media.NewInstance());
            description.addElement(element);
        }
        monitor.worked(40);

        return postExecute(description);
    }

    /**
     * Creates the description and attaches it to the operated element.
     */
    protected abstract void initDescription();

    /** {@inheritDoc} */
    @Override
    public IStatus redo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {

    	element.addDescription(description);

    	return postExecute(description);
    }

    /** {@inheritDoc} */
    @Override
    public IStatus undo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {

    	element.removeDescription(description);

    	return postExecute(null);
    }

}
