/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.operation;

import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.model.common.CdmBase;

/**
 * Implementors may register to operations. The postOperation method will get
 * called after any post operation
 * 
 * @author n.hoffmann
 * @created 24.03.2009
 * @version 1.0
 */
public interface IPostOperationEnabled {

	/**
	 * <p>
	 * This method will be called after executing an operation. If operation
	 * created a new object and the {@link IPostOperationEnabled} has to know
	 * about the object it should be passed to the {@link IPostOperationEnabled}
	 * </p>
	 * 
	 * @param objectAffectedByOperation
	 *            a newly created object for example. May be <code>null</code>
	 * @return true if the method executed successfully. Returning false will
	 *         set the operations status to {@link Status.CANCEL_STATUS} which
	 *         might not be what you want.
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation);
	
	/**
	 * This method will get called after the execution took place and occasional
	 * cleanups were performed. This is typically the last thing to happen.
	 * 
	 * @return a boolean.
	 */
	public boolean onComplete();
	
}
