// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.operation;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.ErrorDialog;

import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;

/**
 * @author cmathew
 * @date 16 Jun 2015
 *
 */
public abstract class RemotingCdmHandler extends AbstractHandler {

    private final String label;

    public RemotingCdmHandler(String label) {
        this.label = label;
    }

    /* (non-Javadoc)
     * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
     */
    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        IStatus allowStatus = allowOperations(event);
        if(allowStatus.isOK()) {
            AbstractOperation op = prepareOperation(event);
            if(op != null) {
                AbstractUtility.executeOperation(op, this);
            }
        } else if(allowStatus.getSeverity() == IStatus.ERROR ||
                allowStatus.getSeverity() == IStatus.WARNING ||
                allowStatus.getSeverity() == IStatus.INFO) {
            MessagingUtils.warningDialog("Can not perform " + label, event.getTrigger(), allowStatus);
        }
        return null;
    }

    public void postOperation(IStatus status) {

        switch(status.getSeverity()) {
        case IStatus.WARNING:
            ErrorDialog.openError(null, "Operation successful but with warnings", null, status);
            break;
        case IStatus.ERROR:
            ErrorDialog.openError(null, "Error executing operation", null, status);
            break;
        default:
        }

        onComplete();
    }

    public abstract IStatus allowOperations(ExecutionEvent event);

    public abstract AbstractOperation prepareOperation(ExecutionEvent event);

    public abstract void onComplete();

}
