// $Id$
/**
 * Copyright (C) 2015 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.operation;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.api.application.CdmChangeEvent.Action;
import eu.etaxonomy.cdm.api.service.UpdateResult;

/**
 * @author cmathew
 * @date 16 Jun 2015
 *
 */
public abstract class RemotingCdmUpdateOperation extends RemotingCdmOperation {

    private UpdateResult updateResult;
    public static boolean throwExceptions = false;

    /**
     * @param label
     */
    public RemotingCdmUpdateOperation(String label, Action action, Object source, boolean async) {
        super(label, action, source, async);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmOperation#doExecute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
     */
    @Override
    protected boolean doExecute(IProgressMonitor monitor, IAdaptable info) {
        try {
            updateResult = doUpdateExecute(monitor, info);
        } catch (Exception e) {
            if(throwExceptions) {
                throw new RuntimeException(e);
            } else {
                UpdateResult exceptionResult = new UpdateResult();
                exceptionResult.addException(e);
                exceptionResult.setAbort();
                updateResult = exceptionResult;
            }
        }
        return updateResult.isOk();
    }

    protected abstract UpdateResult doUpdateExecute(IProgressMonitor monitor, IAdaptable info) throws Exception;

    @Override
    protected void postExecute(boolean success) {
        if(success && updateResult != null) {
            fireDataChangeEvent(updateResult);
        }

    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmOperation#onComplete(boolean)
     */
    @Override
    protected IStatus onComplete(boolean success) {

        if(updateResult != null) {
            int statusFlag = IStatus.OK;

            Collection<Exception> exceptions = updateResult.getExceptions();
            StringBuffer statusMsg = new StringBuffer();
            if(success && updateResult.isOk()) {
                if(exceptions.isEmpty()) {
                    return Status.OK_STATUS;
                } else {
                    statusFlag = IStatus.WARNING;
                    statusMsg.append(getLabel() + " executed sucessfully but with warnings." + System.lineSeparator());
                }
            } else if (updateResult.isError()) {
                statusFlag = IStatus.ERROR;
                statusMsg.append(getLabel() + " failed." + System.lineSeparator());
            } else if (updateResult.isAbort()) {
                statusFlag = IStatus.ERROR;
                statusMsg.append(getLabel() + " aborted." + System.lineSeparator());
            }

            Status[] childStatus = new Status[exceptions.size()];
            int count = 0;
            Set<String> messages = new HashSet<String>();
            for(Exception ex : exceptions) {
                Status status = new Status(statusFlag,
                        "unknown",
                        statusFlag,
                        ex.getLocalizedMessage(),
                        ex);
                messages.add(ex.getLocalizedMessage());
                childStatus[count] = status;
                count++;
            }

            statusMsg.append("Please click on the 'Details' button to see all the errors / warnings");

            MultiStatus multiStatus = new MultiStatus("unknown",
                    statusFlag,
                    childStatus,
                    statusMsg.toString(),
                    null);
            return multiStatus;
        }
        return null;
    }
}
