// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference.menu;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.taxeditor.session.ICdmEntitySession;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author cmathew
 * @date 30 Jul 2015
 *
 */
public abstract class CdmPreferencePage extends PreferencePage implements ICdmEntitySessionEnabled {

    private ICdmEntitySession previousCdmEntitySession;
    private boolean isApply;

    public CdmPreferencePage() {
        initSession();
    }

    public CdmPreferencePage(String title) {
        super(title);
        initSession();
    }

    public CdmPreferencePage(String title, ImageDescriptor image) {
        super(title, image);
        initSession();
    }

    private void initSession() {
        previousCdmEntitySession = CdmStore.getCurrentSessionManager().getActiveSession();
        bindNullSession();
    }

    private void bindNullSession() {
        if(CdmStore.isActive()) {
            CdmStore.getCurrentSessionManager().bindNullSession();
        }
    }

    @Override
    public boolean performOk() {
        if(previousCdmEntitySession != null && !isApply) {
            previousCdmEntitySession.bind();
        }
        return super.performOk();
    }

    @Override
    public boolean performCancel() {
        if(previousCdmEntitySession != null) {
            previousCdmEntitySession.bind();
        }
        return super.performCancel();
    }

    @Override
    public void performApply() {
        isApply = true;
        super.performApply();
        isApply = false;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled#getCdmEntitySession()
     */
    @Override
    public ICdmEntitySession getCdmEntitySession() {
        return CdmStore.getCurrentSessionManager().getNullSession();
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled#getRootEntities()
     */
    @Override
    public <T extends CdmBase> Collection<T> getRootEntities() {
        return null;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled#getPropertyPathsMap()
     */
    @Override
    public Map<Object, List<String>> getPropertyPathsMap() {
        return null;
    }

}
