// $Id$
/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.preference.wizard;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.UUID;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.OrderedTermVocabulary;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author alex
 * @date 21.07.2014
 *
 */
public class AvailableDistributionWizard extends Wizard implements IConversationEnabled {

    private ConversationHolder conversation;
    private final AvailableDistributionPage aPage;

    public AvailableDistributionWizard() {
        setWindowTitle("Distribution Selection Wizard");
        // TODO if preferenceStore has elements checked load elements in wizard
        aPage = new AvailableDistributionPage("AvailableDistributionPage");
    }

    /*
     * (non-Javadoc)
     *
     * @see org.eclipse.jface.wizard.Wizard#performFinish()
     */
    @Override
    public boolean performFinish() {
        // TODO: get Selection and save in EditorPreferences
        if (!checkNoneChecked()) {
            Object[] checkedElements = aPage.getViewer().getCheckedElements();
            Object[] grayedElements = aPage.getViewer().getGrayedElements();
            ArrayList checkedList = new ArrayList(Arrays.asList(checkedElements));
            ArrayList grayedList = new ArrayList(Arrays.asList(grayedElements));
            checkedList = (ArrayList) CollectionUtils.subtract(checkedList, grayedList);
            ArrayList<UUID> listUIIDChecked = new ArrayList<UUID>();
            ArrayList<UUID> listUIIDGrayed = new ArrayList<UUID>();
            for (Object o : checkedList) {
                if (o instanceof NamedArea) {
                    NamedArea na = (NamedArea) o;
                    listUIIDChecked.add(na.getUuid());

                }else if(o instanceof OrderedTermVocabulary){
                    OrderedTermVocabulary otb = (OrderedTermVocabulary) o;
                    listUIIDGrayed.add(otb.getUuid());
                }
            }
            for (Object o : grayedList) {
                if(o instanceof DefinedTermBase){
                    DefinedTermBase dt = (DefinedTermBase) o;
                    listUIIDGrayed.add(dt.getUuid());
                }else if(o instanceof OrderedTermVocabulary){
                    OrderedTermVocabulary otb = (OrderedTermVocabulary) o;
                    listUIIDGrayed.add(otb.getUuid());
                }

            }

            String saveCheckedElements = StringUtils.join(listUIIDChecked, ",");
            String saveGrayedElements = StringUtils.join(listUIIDGrayed, ",");
            IPreferenceStore preferenceStore = PreferencesUtil.getPreferenceStore();
            preferenceStore.setValue(PreferencesUtil.DISTRIBUTION_AREA_OCCURENCE_STATUS, saveCheckedElements);
            preferenceStore.setValue(PreferencesUtil.DISTRIBUTION_AREA_OCCURENCE_STATUS_GRAYED, saveGrayedElements);
            return true;
        } else {
            return false;
        }
    }

    @Override
    public void addPages() {
        addPage(aPage);
    }

    private boolean checkNoneChecked() {

        if (aPage.getViewer().getCheckedElements().length == 0) {
            aPage.setMessage("Please check at least one item", aPage.WARNING);
            return true;
        } else {
            aPage.setMessage(null);
            return false;
        }
    }

    @Override
    public ConversationHolder getConversationHolder() {
        if (conversation == null) {
            conversation = CdmStore.createConversation();
        }
        return conversation;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update
     * (eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
     */
    @Override
    public void update(CdmDataChangeMap changeEvents) {
        // TODO Auto-generated method stub

    }
}
