// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.deleteConfigurator;

import java.util.List;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.PojoProperties;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;

import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.widgets.FormToolkit;

import eu.etaxonomy.cdm.api.service.IClassificationService;
import eu.etaxonomy.cdm.api.service.config.TaxonDeletionConfigurator;

import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.taxeditor.store.CdmStore;


/**
 * @author pplitzner
 * @date Feb 18, 2015
 *
 */
public class DeleteTaxonConfiguratorComposite extends Composite implements SelectionListener{
    private final DataBindingContext m_bindingContext;

    private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());
    private final TaxonDeletionConfigurator configurator;
    private final Button btnDeleteTaxonName;
    private final Button btnDeleteInAllClassifications;
    private Combo classificationSelectionCombo;
    private 
    
    Classification selectedClassification;
    private List<Classification> classifications;
    

    /**
     * Create the composite.
     * @param parent
     * @param style
     */
    public DeleteTaxonConfiguratorComposite(TaxonDeletionConfigurator configurator, Composite parent, int style) {
        super(parent, style);
        this.configurator = configurator;
        addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                toolkit.dispose();
            }
        });
        toolkit.paintBordersFor(this);
        setLayout(new GridLayout());
        setBackground(getBackground());

        btnDeleteTaxonName = new Button(this, SWT.CHECK);
        btnDeleteTaxonName.setText("Delete taxon name if possible");
        btnDeleteTaxonName.setSelection(true);
        
        btnDeleteInAllClassifications = new Button(this, SWT.CHECK);
        btnDeleteInAllClassifications.setText("Delete taxon in all classifications");
        btnDeleteInAllClassifications.setSelection(true);
        
       // createClassificationSelectionCombo(this);
        
        m_bindingContext = initDataBindings();

    }

    protected DataBindingContext initDataBindings() {
        DataBindingContext bindingContext = new DataBindingContext();
        //
        IObservableValue observeSelectionBtnDeleteTaxonNameObserveWidget = WidgetProperties.selection().observe(btnDeleteTaxonName);
        IObservableValue deleteNameIfPossibleConfiguratorObserveValue = PojoProperties.value("deleteNameIfPossible").observe(configurator);
        
        IObservableValue observeSelectionBtnDeleteInAllClassificationObserveWidget = WidgetProperties.selection().observe(btnDeleteInAllClassifications);
        IObservableValue deleteInAllClassificationsConfiguratorObserveValue = PojoProperties.value("deleteInAllClassifications").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnDeleteTaxonNameObserveWidget, deleteNameIfPossibleConfiguratorObserveValue, null, null);
        bindingContext.bindValue(observeSelectionBtnDeleteInAllClassificationObserveWidget, deleteInAllClassificationsConfiguratorObserveValue, null, null);
        
        
        
        //
        return bindingContext;
    }
    
  /* private Control createClassificationSelectionCombo(Composite parent){
//		classifications = CdmStore.getTaxonTreeService().list(null, null, null, null, null);

		Composite classificationSelection = new Composite(parent, SWT.NULL);
		classificationSelection.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, true));

		GridLayout layout = new GridLayout();
		classificationSelection.setLayout(layout);

		Label label = new Label(classificationSelection, SWT.NULL);
		// TODO not working is not really true but leave it here to remind everyone that this is under construction
		label.setText("Select Classification");
		classificationSelectionCombo = new Combo(classificationSelection, SWT.BORDER | SWT.READ_ONLY);
		classificationSelectionCombo.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, true));
		classifications = CdmStore.getService(IClassificationService.class).list(null, null, null, null, null);

		for(Classification tree : classifications){
			classificationSelectionCombo.add(tree.getName().getText(), classifications.indexOf(tree));

		}

		classificationSelectionCombo.select(0);
		classificationSelectionCombo.addSelectionListener(this);
		
		return classificationSelection;
	}*/

@Override
public void widgetSelected(SelectionEvent e) {
	selectedClassification = classifications.get(classificationSelectionCombo.getSelectionIndex());
}

@Override
public void widgetDefaultSelected(SelectionEvent e) {
	// TODO Auto-generated method stub
	
}
}
