// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.taxeditor.Messages;

/**
 *
 * @author pplitzner
 * @date Sep 21, 2015
 *
 */
public abstract class AbstractUriWithExceptionLabelElement <T> extends TextWithLabelElement {

    protected Label labelException;

	protected AbstractUriWithExceptionLabelElement(CdmFormFactory formFactory,
			ICdmFormElement parentElement, String labelString,
			T initialObject, Integer textHeight, int style) {
		super(formFactory, parentElement, false);

        init(formFactory, labelString, initialObject, textHeight, style);
    }

    protected void init(CdmFormFactory formFactory, String labelString, T initialObject, Integer textHeight, int style) {
        //label
        initLabel(formFactory, labelString, false, getLayoutComposite());

        //uri text
        initText(formFactory, null, textHeight, null, false, style, getLayoutComposite());

        //exceptionLabel
        initExceptionLabel(getLayoutComposite(), formFactory, initialObject);
    }

    protected void initExceptionLabel(Composite parent, CdmFormFactory formFactory, T initialObject) {
        labelException = formFactory.createLabel(parent, "", SWT.WRAP); //$NON-NLS-1$
		int numColumns = AbstractFormSection.DEFAULT_NUM_COLUMNS;
		if(parent.getLayout() instanceof TableWrapLayout){
		    numColumns = ((TableWrapLayout)parent.getLayout()).numColumns;
		}
        labelException.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(numColumns, 1));
		addControl(labelException);
		setParsedText(initialObject);
    }


    public abstract void setParsedText(T object);

    protected abstract T getParsedText() throws Exception;

    public T parseText(){
        try {
            labelException.setFont(JFaceResources.getFontRegistry().get(JFaceResources.DEFAULT_FONT));
            labelException.setForeground(getPersistentBackground());
            labelException.setText(""); //$NON-NLS-1$
            return getParsedText();
        } catch (Exception e) {
            labelException.setFont(JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT));
            labelException.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
            labelException.setText(Messages.UriWithLabelElement_URL_NOT_SAVED+e.getLocalizedMessage());
            return null;
        }
    }

}
