// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.swt.events.ModifyEvent;

import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>LanguageStringWithLabelElement class.</p>
 *
 * @author n.hoffmann
 * @created Mar 25, 2010
 * @version 1.0
 */
public class LanguageStringWithLabelElement extends TextWithLabelElement {

	private LanguageString languageString;

	/**
	 * <p>Constructor for LanguageStringWithLabelElement.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param labelString a {@link java.lang.String} object.
	 * @param languageString a {@link eu.etaxonomy.cdm.model.common.LanguageString} object.
	 * @param style a int.
	 */
	protected LanguageStringWithLabelElement(CdmFormFactory formFactory,
			ICdmFormElement parentElement, String labelString,
			LanguageString languageString, int style) {
		this(formFactory, parentElement, labelString, languageString, null, style);
	}

	/**
	 * <p>Constructor for LanguageStringWithLabelElement.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param labelString a {@link java.lang.String} object.
	 * @param languageString a {@link eu.etaxonomy.cdm.model.common.LanguageString} object.
	 * @param height a int.
	 * @param style a int.
	 */
	protected LanguageStringWithLabelElement(CdmFormFactory formFactory,
			ICdmFormElement parentElement, String labelString,
			LanguageString languageString, Integer height, int style) {
		this(formFactory, parentElement, labelString, languageString, height, true, style);
	}

	protected LanguageStringWithLabelElement(CdmFormFactory formFactory,
	        ICdmFormElement parentElement, String labelString,
	        LanguageString languageString, Integer height, boolean isMultiLine, int style) {
	    super(formFactory, parentElement, labelString, null, height, isMultiLine, style);

	    setLanguageString(languageString);

	}

	/**
	 * <p>Getter for the field <code>languageString</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.common.LanguageString} object.
	 */
	public LanguageString getLanguageString() {
		return languageString;
	}

	public LanguageString updateLanguageString(LanguageString ls) {
	    if(ls == null){
            ls = LanguageString.NewInstance("", CdmStore.getDefaultLanguage());
        }
	    ls.setText(text.getText());
	    return ls;
	}

	/** {@inheritDoc} */
	@Override
	public void modifyText(ModifyEvent e) {
		if(languageString == null){
			languageString = LanguageString.NewInstance("", CdmStore.getDefaultLanguage());
		}

		languageString.setText(text.getText());
		super.modifyText(e);
	}

	/** {@inheritDoc} */
	@Override
	public void setText(String text) {
		if(text != null){
			languageString.setText(text);
			super.setText(text);
		}
	}

	/**
	 * <p>Setter for the field <code>languageString</code>.</p>
	 *
	 * @param languageString a {@link eu.etaxonomy.cdm.model.common.LanguageString} object.
	 */
	public void setLanguageString(LanguageString languageString) {
		this.languageString = languageString;
		super.setText(languageString != null ? languageString.getText() : "");
	}
}
