// $Id$
/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.mvc.element;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.joda.time.DateTime;

import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.mvc.AbstractCdmComposite;

/**
 * @author pplitzner
 * @date 16.06.2014
 *
 */
public class DateElement extends AbstractCdmComposite<DateElementController> {

    private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());
    private final DateTime initialDateTime;
    private final Text textDate;

    /**
     * Create the composite.
     * @param parent
     * @param style
     */
    public DateElement(Composite parent, DateTime initialDateTime, int style) {
        super(parent, style);
        this.initialDateTime = initialDateTime;
        addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                toolkit.dispose();
            }
        });
        toolkit.adapt(this);
        toolkit.paintBordersFor(this);
        {
            TableWrapLayout tableWrapLayout = new TableWrapLayout();
            tableWrapLayout.verticalSpacing = 0;
            tableWrapLayout.topMargin = 0;
            tableWrapLayout.rightMargin = 0;
            tableWrapLayout.leftMargin = 0;
            tableWrapLayout.horizontalSpacing = 0;
            tableWrapLayout.bottomMargin = 0;
            setLayout(tableWrapLayout);
        }

        textDate = new Text(this, SWT.BORDER | SWT.WRAP);
        textDate.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.TOP, 1, 1));
        toolkit.adapt(textDate, true, true);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.mvc.AbstractCdmComposite#initInternalController(eu.etaxonomy.taxeditor.ui.element.CdmFormFactory, eu.etaxonomy.taxeditor.ui.element.ICdmFormElement)
     */
    @Override
    protected void initInternalController(CdmFormFactory formFactory, ICdmFormElement parentElement) {
        controller = new DateElementController(this, formFactory, parentElement, initialDateTime, SWT.NONE);
        formFactory.adapt(controller);
    }
    public Text getTextDate() {
        return textDate;
    }
}
