// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.password;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.core.AuthenticationException;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;

/**
 * @author n.hoffmann
 * @created Mar 10, 2011
 * @version 1.0
 */
public class PasswordWizard extends Wizard implements IConversationEnabled{

	private PasswordWizardPage passwordPage;
	private final User user;
	private final ConversationHolder conversation;

	/**
	 * @param user the user to change the password for
	 * @param conversation this optional parameter can be null. Only supply conversation if you
	 * need to run this wizard is a nested conversation
	 */
	public PasswordWizard(User user, ConversationHolder conversation){
		this.conversation = conversation;
		this.user = user;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	@Override
	public void addPages() {
		CdmFormFactory formFactory = new CdmFormFactory(Display.getDefault());

		passwordPage = new PasswordWizardPage(formFactory, getConversationHolder(), user);
		addPage(passwordPage);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {

		ConversationHolder internalConversation = CdmStore.getCurrentApplicationConfiguration().NewConversation();
	    internalConversation.bind();
	    internalConversation.startTransaction();
		try{
			if(passwordPage.isChangingOwnPassword()){
			    // change own password with validating of old one
				CdmStore.getService(IUserService.class).changePassword(passwordPage.getOldPassword(), passwordPage.getNewPassword());
			}else{
			    // change others passwords
			    CdmStore.getService(IUserService.class).changePasswordForUser(user.getUsername(), passwordPage.getNewPassword());
			}
			CdmStore.getService(IUserService.class).loadWithUpdate(user.getUuid());
		    internalConversation.commit(false);
		    internalConversation.unbind();
		    internalConversation.close();
			return true;
		}catch(AccessDeniedException e){
			MessagingUtils.warningDialog("Could not change password", this, "The old password is not correct.");
			return false;
		}catch (AuthenticationException e){
			MessagingUtils.warningDialog("Could not change password", this, "The old password is not correct.");

			return false;
		}catch(Exception e){
			MessagingUtils.warningDialog("Problem with changing password", this, "The password could not be changed. " + e.getMessage());
			return false;
		}finally{
		    if(conversation != null){
		        conversation.bind();
		    }
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	@Override
	public void update(CdmDataChangeMap arg0) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	@Override
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

}
