// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.description.detail;

import org.eclipse.swt.SWT;

import eu.etaxonomy.cdm.model.description.TaxonInteraction;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * <p>
 * TaxonInteractionDetailElement class.
 * </p>
 * 
 * @author n.hoffmann
 * @created Jun 10, 2010
 * @version 1.0
 */
public class TaxonInteractionDetailElement extends
		AbstractDetailedDescriptionDetailElement<TaxonInteraction> {

	private EntitySelectionElement<Taxon> taxonSelection;
	private TextWithLabelElement interactionDescription;

	/**
	 * <p>
	 * Constructor for TaxonInteractionDetailElement.
	 * </p>
	 * 
	 * @param formFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param formElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 * @param entity
	 *            a {@link eu.etaxonomy.cdm.model.description.TaxonInteraction}
	 *            object.
	 * @param style
	 *            a int.
	 */
	public TaxonInteractionDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement, TaxonInteraction entity, int style) {
		super(formFactory, formElement, entity, style);
	}

	/** {@inheritDoc} */
	@Override
	protected void createControls(ICdmFormElement formElement,
			TaxonInteraction entity, int style) {
		taxonSelection = formFactory
				.createSelectionElement(Taxon.class,
						getConversationHolder(), this, "Taxon",
						entity.getTaxon2(), EntitySelectionElement.NOTHING,
						SWT.WRAP);

		interactionDescription = formFactory.createMultiLineTextWithLabel(this,
				"Relationship to the taxon:", 200, SWT.WRAP);
		interactionDescription.setText(entity.getDescription(CdmStore
				.getDefaultLanguage()));
	}

	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == taxonSelection) {
			getEntity().setTaxon2(taxonSelection.getEntity());
		} else if (eventSource == interactionDescription) {
			getEntity().putDescription(CdmStore.getDefaultLanguage(), interactionDescription.getText());
		}
	}
}
