// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.identifier;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.common.DefinedTerm;
import eu.etaxonomy.cdm.model.common.Identifier;
import eu.etaxonomy.cdm.model.common.TermType;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;

/**
 * <p>
 * CollectionDetailElement class.
 * </p>
 *
 * @author n.hoffmann
 * @created Oct 13, 2010
 * @version 1.0
 */
public class IdentifierDetailElement extends AbstractEntityCollectionElement<Identifier> {

	private TextWithLabelElement textIdentifier;

	private TermComboElement<DefinedTerm> comboIdentifierType;

	/**
     * <p>
     * Constructor for CollectingAreasDetailElement.
     * </p>
     *
     * @param formFactory
     *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
     *            object.
     * @param section
     *            a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection}
     *            object.
     * @param entity
     *            a {@link eu.etaxonomy.cdm.model.location.NamedArea} object.
     * @param removeListener
     *            a {@link org.eclipse.swt.events.SelectionListener} object.
     * @param style
     *            a int.
     */
    public IdentifierDetailElement(CdmFormFactory formFactory,
            AbstractFormSection section, Identifier entity,
            SelectionListener removeListener, int style) {
        super(formFactory, section, entity, removeListener, null, style);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#setEntity
     * (java.lang.Object)
     */
    /** {@inheritDoc} */
    @Override
    public void setEntity(Identifier entity) {
        this.entity = entity;
        textIdentifier.setText(entity.getIdentifier());
        comboIdentifierType.setSelection(entity.getType());
    }

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#createControls
	 * (eu.etaxonomy.taxeditor.forms.ICdmFormElement,
	 * eu.etaxonomy.cdm.model.common.IAnnotatableEntity, int)
	 */
	/** {@inheritDoc} */
	@Override
    public void createControls(ICdmFormElement formElement, int style) {
		textIdentifier = formFactory.createTextWithLabelElement(formElement, "Identifier", null, style);
		comboIdentifierType = formFactory.createDefinedTermComboElement(TermType.IdentifierType, formElement, "Type", null, style);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleEvent(java
	 * .lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == textIdentifier) {
			getEntity().setIdentifier(textIdentifier.getText());
		} else if (eventSource == comboIdentifierType) {
			getEntity().setType(comboIdentifierType.getSelection());
		}
	}

}
