/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.key;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.KeyStatementElement;
import eu.etaxonomy.taxeditor.ui.element.NumberWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author n.hoffmann
 *
 */
public class PolytomousKeyNodeDetailElement extends
		AbstractCdmDetailElement<PolytomousKeyNode> {

	private KeyStatementElement element_statement;

	private EntitySelectionElement<Feature> selection_feature;

	private EntitySelectionElement<Taxon> selection_taxon;

	private EntitySelectionElement<PolytomousKey> selection_subkey;

	private EntitySelectionElement<PolytomousKeyNode> selection_otherNode;

	private KeyStatementElement element_question;

	public PolytomousKeyNodeDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	@Override
	protected void createControls(ICdmFormElement formElement,
			PolytomousKeyNode entity, int style) {
		if (!entity.isLeaf()) {
			NumberWithLabelElement number_nodeNumber = formFactory
					.createNumberTextWithLabelElement(formElement,
							"Node Number", entity.getNodeNumber(), style);
			number_nodeNumber.setEnabled(false);
		}

		if (entity.getParent() != null) {
			PolytomousKeyNode parent = entity.getParent();

			element_question = formFactory.createKeyStatementElement(
					formElement, "Question", parent.getQuestion(), 50, style);

			selection_feature = formFactory
					.createSelectionElement(Feature.class,
							getConversationHolder(), formElement, "Character",
							parent.getFeature(),
							EntitySelectionElement.EDITABLE | EntitySelectionElement.DELETABLE, style);
		}

		element_statement = formFactory.createKeyStatementElement(formElement,
				"State(ment)", entity.getStatement(), 50, style);

		if (entity.isLeaf()) {
			selection_taxon = formFactory
					.createSelectionElement(Taxon.class,
							getConversationHolder(), formElement, "Taxon",
							entity.getTaxon(), EntitySelectionElement.NOTHING,
							style);
		} else{
			selection_taxon = formFactory
					.createSelectionElement(Taxon.class,
							getConversationHolder(), formElement, "Taxon",
							entity.getTaxon(), EntitySelectionElement.DELETABLE,
							style);
		}

		selection_subkey = formFactory
				.createSelectionElement(PolytomousKey.class,
						getConversationHolder(), formElement, "Subkey",
						entity.getSubkey(),
						EntitySelectionElement.EDITABLE | EntitySelectionElement.DELETABLE, style);

		selection_otherNode = formFactory
				.createSelectionElement(PolytomousKeyNode.class,
						getConversationHolder(), formElement, "Other Node",
						entity.getOtherNode(),
						EntitySelectionElement.DELETABLE, style);

		formFactory.createMultiLanguageTextElement(formElement,
				"Modifying Text", entity.getModifyingText(), 50, style);

	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleEvent(java
	 * .lang.Object)
	 */
	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == element_question) {
			getEntity().getParent().setQuestion(element_question.updateKeyStatement(getEntity().getQuestion()));
		} else if (eventSource == element_statement) {
		    getEntity().setStatement(element_statement.updateKeyStatement(getEntity().getStatement()));
		} else if (eventSource == selection_feature) {
			getEntity().getParent().setFeature(selection_feature.getEntity());
		} else if (eventSource == selection_taxon) {
			getEntity().setTaxon(selection_taxon.getEntity());
		} else if (eventSource == selection_subkey) {
			getEntity().setSubkey(selection_subkey.getEntity());
		} else if (eventSource == selection_otherNode) {
			getEntity().setOtherNode(selection_otherNode.getEntity());
		}
	}
}
