// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.media;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;

import eu.etaxonomy.taxeditor.model.ImageResources;

/**
 * Adds a switch to a section with an {@link ITogglableMediaElement}
 * to allow switching between simple and advanced media view
 * @author pplitzner
 * @date Jun 29, 2015
 *
 */
public class MediaViewSwitch {

    /**
     * Creates an {@link IAction} for the given {@link ITogglableMediaElement}
     * that switches between the two states of the element.
     */
    public static IAction createSwitch(final ITogglableMediaElement togglableMediaElement) {
        final String label = "Change View Type";

        Action switchAction = new Action(label, ImageResources.getImageDescriptor(ImageResources.SWITCH_VIEW_TYPE)) {
            @Override
            public void run() {
                if(togglableMediaElement!=null){
                    togglableMediaElement.toggleAdvancedMediaView();
                }
            }
        };
        switchAction.setToolTipText(label);

        return switchAction;
    }
}
