// $Id$
/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.name;

import java.util.Collection;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.wizard.WizardDialog;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationship;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;

/**
 * @author pplitzner
 * @date 27.01.2014
 *
 */
public class SynonymRelationshipDetailSection extends AbstractEntityCollectionSection<Synonym, SynonymRelationship> {

    /**
     * <p>Constructor for NameRelationshipDetailSection.</p>
     *
     * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
     * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
     * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
     * @param style a int.
     */
    public SynonymRelationshipDetailSection(CdmFormFactory formFactory, ConversationHolder conversation,
            ICdmFormElement parentElement, int style) {
        super(formFactory, conversation, parentElement, "Synonym Relationship", style);
    }

    /** {@inheritDoc} */
    @Override
    public void addElement(SynonymRelationship element) {
        // nothing
    }

    /** {@inheritDoc} */
    @Override
    public SynonymRelationship createNewElement() {

        SynonymRelationshipWizard wizard = new SynonymRelationshipWizard(this);
        WizardDialog dialog = new WizardDialog(StoreUtil.getShell(), wizard);
        int status = dialog.open();

        if(status == IStatus.OK) {
            return wizard.getSynonymRelationship();
        }

        return null;
    }

    /** {@inheritDoc} */
    @Override
    public Collection<SynonymRelationship> getCollection(Synonym entity) {
        Collection<SynonymRelationship> allSynonymRelationships = entity.getSynonymRelations();
        return allSynonymRelationships;
    }

    /** {@inheritDoc} */
    @Override
    public String getEmptyString() {
        return "No synonym relationships yet.";
    }

    /** {@inheritDoc} */
    @Override
    protected String getTooltipString() {
        return "Add a new synonym relationship from this name.";
    }

    /** {@inheritDoc} */
    @Override
    public void removeElement(SynonymRelationship element) {
        getEntity().removeSynonymRelation(element);
    }

}
