// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.occurrence.derivedUnit;

import org.eclipse.ui.forms.widgets.ExpandableComposite;

import eu.etaxonomy.cdm.model.common.DefinedTerm;
import eu.etaxonomy.cdm.model.common.TermType;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.PreservationMethod;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.SpecimenCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * <p>
 * DerivedUnitBaseDetailElement class.
 * </p>
 *
 * @author n.hoffmann
 * @created Jun 24, 2010
 * @version 1.0
 */
public class PreservedSpecimenDetailElement extends
		AbstractCdmDetailElement<DerivedUnit> {

	private TextWithLabelElement text_derivedUnitDefinitions;

	private TextWithLabelElement text_barcode;

	private TextWithLabelElement text_catalogNumber;

	private EntitySelectionElement<TaxonNameBase> selection_storedUnder;

	private TextWithLabelElement text_exsicatum;

	private TermComboElement<DefinedTerm> combo_preservationMethod;

	private SpecimenCollectionDetailSection section_duplicates;

	private PreservedSpecimenSourceCollectionDetailSection section_source;

	/**
	 * <p>
	 * Constructor for DerivedUnitBaseDetailElement.
	 * </p>
	 *
	 * @param formFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param formElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 */
	public PreservedSpecimenDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	/** {@inheritDoc} */
	@Override
    protected void createControls(ICdmFormElement formElement, DerivedUnit entity, int style) {
		// Disable for now
		// text_derivedUnitDefinitions =
		// formFactory.createTextWithLabelElement(formElement,
		// "Specimen Definition",
		// facade.getDerivedUnitDefinition(CdmStore.getDefaultLanguage()),
		// style);
        text_barcode = formFactory.createTextWithLabelElement(formElement, "Barcode", entity.getBarcode(), style);
        text_catalogNumber = formFactory.createTextWithLabelElement(formElement, "Catalog Number", entity.getCatalogNumber(), style);


        PreservationMethod preservationMethod = entity.getPreservation();
        combo_preservationMethod = formFactory.createDefinedTermComboElement(TermType.MaterialOrMethod, formElement, "Preservation Method", preservationMethod==null?null:preservationMethod.getDefinedMaterialOrMethod(), style);
        selection_storedUnder = formFactory.createSelectionElement(TaxonNameBase.class, getConversationHolder(), formElement, "Stored Under", entity.getStoredUnder(), EntitySelectionElement.ALL, style);

        text_exsicatum = formFactory.createTextWithLabelElement(formElement, "Exsiccatum", entity.getExsiccatum(), style);

        section_source = formFactory.createPreservedSpecimenSourceCollectionDetailSection(getConversationHolder(), formElement, ExpandableComposite.TWISTIE);
		section_source.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
		section_source.setEntity(entity);
	}

    /** {@inheritDoc} */
    @Override
    public void handleEvent(Object eventSource) {
        if (eventSource == text_derivedUnitDefinitions) {
            getEntity().putDefinition(CdmStore.getDefaultLanguage(), text_derivedUnitDefinitions.getText());
        } else if (eventSource == text_barcode) {
            getEntity().setBarcode(text_barcode.getText());
        } else if (eventSource == text_catalogNumber) {
            getEntity().setCatalogNumber(text_catalogNumber.getText());
        } else if (eventSource == combo_preservationMethod) {
            PreservationMethod preservationMethod = getEntity().getPreservation();
            if(preservationMethod!=null){
                preservationMethod.setDefinedMaterialOrMethod(combo_preservationMethod.getSelection());
            }
        } else if (eventSource == selection_storedUnder) {
            getEntity().setStoredUnder(selection_storedUnder.getEntity());
        } else if (eventSource == text_exsicatum) {
            try {
                getEntity().setExsiccatum(text_exsicatum.getText());
            } catch (Exception e) {
                MessagingUtils.error(this.getClass(), e);
            }
        }
    }
}
