// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.reference;

import org.eclipse.jface.viewers.ISelectionProvider;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.reference.INomenclaturalReference;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.ui.section.ITaxonBaseDetailSection;

/**
 * <p>NomenclaturalReferenceDetailSection class.</p>
 *
 * @author n.hoffmann
 * @created Mar 12, 2010
 * @version 1.0
 */
public class NomenclaturalReferenceDetailSection extends
		AbstractCdmDetailSection<TaxonNameBase>  implements ITaxonBaseDetailSection{

	private TaxonBase taxonBase;

	/**
	 * <p>Constructor for NomenclaturalReferenceDetailSection.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param selectionProvider a {@link org.eclipse.jface.viewers.ISelectionProvider} object.
	 * @param style a int.
	 */
	public NomenclaturalReferenceDetailSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation,
			ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style) {
		super(cdmFormFactory, conversation, parentElement, selectionProvider, style);
	}

	/** {@inheritDoc} */
	@Override
	public String getHeading() {
		return "Nomenclatural Reference";
	}

	/** {@inheritDoc} */
	@Override
    public void setTaxonBase(TaxonBase entity) {
		this.taxonBase = entity;
		TaxonNameBase name = (TaxonNameBase) HibernateProxyHelper.deproxy(entity.getName());
		setEntity(name);
	}

	/** {@inheritDoc} */
	@Override
	protected void setSectionTitle() {
		String title = "";
		if(getEntity() != null && getEntity().getNomenclaturalReference() != null){
			INomenclaturalReference reference = getEntity().getNomenclaturalReference();
			String nomenclaturalCitation = reference.getNomenclaturalCitation(getEntity().getNomenclaturalMicroReference());
			title = ": " + nomenclaturalCitation;
		}
		this.setText(getHeading() + title);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.ITaxonDetailSection#getTaxon(eu.etaxonomy.cdm.model.taxon.TaxonBase)
	 */
	@Override
	public TaxonBase getTaxonBase() {
		return taxonBase;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection#createCdmDetailElement(eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection, int)
	 */
	@Override
	protected AbstractCdmDetailElement<TaxonNameBase> createCdmDetailElement(AbstractCdmDetailSection<TaxonNameBase> parentElement, int style) {
	    return formFactory.createNomenclaturalReferenceDetailElement(parentElement, style);
	}
}
