/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.supplemental;

import java.util.Collection;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.common.IIdentifiableEntity;
import eu.etaxonomy.cdm.model.common.IdentifiableSource;
import eu.etaxonomy.cdm.model.common.OriginalSourceType;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;

/**
 * <p>SourceSection class.</p>
 *
 * @author n.hoffmann
 * @created Nov 17, 2009
 * @version 1.0
 */
public class SourceSection extends AbstractEntityCollectionSection<IIdentifiableEntity, IdentifiableSource> {

	/**
	 * <p>Constructor for SourceSection.</p>
	 *
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	public SourceSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation, parentElement, "Sources", style);
	}

	/** {@inheritDoc} */
	@Override
	public IdentifiableSource createNewElement() {
	    IIdentifiableEntity entity = getEntity();
	    OriginalSourceType type;
	    if(entity instanceof Media) {
	        type = OriginalSourceType.PrimaryMediaSource;
        }
	    else{
	        //default
	        type = OriginalSourceType.PrimaryTaxonomicSource;
	    }
	    return IdentifiableSource.NewInstance(type);
	}

	/** {@inheritDoc} */
	@Override
	public Collection<IdentifiableSource> getCollection(
			IIdentifiableEntity entity) {
		return entity.getSources();
	}

	/** {@inheritDoc} */
	@Override
	public String getEmptyString() {
		return "No references yet.";
	}

	/** {@inheritDoc} */
	@Override
	protected String getTooltipString() {
		return "Create a new reference.";
	}

	/** {@inheritDoc} */
	@Override
	public void addElement(IdentifiableSource element) {
		getEntity().addSource(element);
	}

	/** {@inheritDoc} */
	@Override
	public void removeElement(IdentifiableSource element) {
		getEntity().removeSource(element);
	}

}
