// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.supplemental;

import org.eclipse.swt.widgets.Label;
import org.joda.time.DateTime;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.common.VersionableEntity;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEntityElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;

/**
 * <p>VersionElement class.</p>
 *
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 */
public class VersionElement extends AbstractCdmFormElement implements IEntityElement<VersionableEntity> {

	private Label label_created;
	private Label label_updated;
	
	
	private VersionableEntity entity;
	
	/**
	 * <p>Constructor for VersionElement.</p>
	 *
	 * @param style a int.
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param entity a {@link eu.etaxonomy.cdm.model.common.VersionableEntity} object.
	 */
	public VersionElement(CdmFormFactory formFactory, ICdmFormElement parentElement, VersionableEntity entity, int style) {
		super(formFactory, parentElement);
			
		label_created = formFactory.createLabel(getLayoutComposite(), null, style);
		label_created.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

		label_updated = formFactory.createLabel(getLayoutComposite(), null, style);
		label_updated.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());
		
		
		setEntity(entity);
	}
	
	/**
	 * <p>Setter for the field <code>entity</code>.</p>
	 *
	 * @param entity a {@link eu.etaxonomy.cdm.model.common.VersionableEntity} object.
	 */
	public void setEntity(VersionableEntity entity){

		this.entity = entity;
		
		String createdString = "";
		String updatedString = "";
		
		if(entity != null){
			createdString = "Created "
					+ dateFormat(entity.getCreated())
					+ " by " + userFormat(entity.getCreatedBy());			
			updatedString = "Updated "
					+ dateFormat(entity.getUpdated())
					+ " by " + userFormat(entity.getUpdatedBy());
		}
		
		label_created.setText(createdString);
		label_updated.setText(updatedString);
	}
	
	private String dateFormat(DateTime dateTime){
		if(dateTime == null){
			return "";
		}
		return CdmUtils.Nz(dateTime.toString(PreferencesUtil.getDateFormatPattern()));
	}
	
	private String userFormat(User user){
		if(user == null){
			return "";
		}
		// TODO agree on what to display username or real name or even more from shibboleth
		return CdmUtils.Nz(user.getUsername());
	}

	/**
	 * <p>Getter for the field <code>entity</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.common.VersionableEntity} object.
	 */
	public VersionableEntity getEntity() {
		return entity;
	}

	@Override
	public void setSelected(boolean selected) {
		// this entity element is not likely to get selected
	}
}
