// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.view;

import java.util.Collection;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.ContextListenerAdapter;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Feb 16, 2011
 * @version 1.0
 */
public abstract class AbstractAccountViewPart<T> extends AbstractCdmViewPart {

	private class ContextListener extends ContextListenerAdapter{

		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.IContextListener#contextStop(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
		 */
		@Override
		public void contextStop(IMemento memento, IProgressMonitor monitor) {
			monitor.subTask("Shutting down User View");
			viewer.setInput(null);
		}

		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.IContextListener#contextStart(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
		 */
		@Override
		public void contextStart(IMemento memento, IProgressMonitor monitor) {
			monitor.subTask("Initiating User View");
			viewer.setInput(getInput());
		}
	}
	
	protected TableViewer viewer;
	
	private IContextListener contextListener;
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		contextListener = new ContextListener();
		CdmStore.getContextManager().addContextListener(contextListener);
		
		FillLayout fillLayout = new FillLayout();
		fillLayout.marginWidth = 0;
		fillLayout.marginHeight = 0;
		fillLayout.type = SWT.VERTICAL;	
		parent.setLayout(fillLayout);
		
		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.FULL_SELECTION);
		
		getSite().setSelectionProvider(viewer);
		
		createColumns(viewer);
		
		MenuManager menuMgr = new MenuManager();
		menuMgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		getSite().registerContextMenu(menuMgr, viewer);

		Control control = viewer.getControl();
		Menu menu = menuMgr.createContextMenu(control);
		control.setMenu(menu);	
		
		viewer.setContentProvider(new ArrayContentProvider());
		viewer.setLabelProvider(getLabelProvider());
				
		if(CdmStore.isActive()){
			viewer.setInput(getInput());
		}
		
		
		this.addListenerObject(new FocusAdapter() {
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.FocusAdapter#focusGained(org.eclipse.swt.events.FocusEvent)
			 */
			@Override
			public void focusGained(FocusEvent e) {
				super.focusGained(e);
				getConversationHolder().bind();
				viewer.getControl().setFocus();
			}
		});
	}
	
	/**
	 * @return
	 */
	protected abstract IBaseLabelProvider getLabelProvider();

	/**
	 * @param viewer2
	 */
	protected abstract void createColumns(TableViewer viewer);
	
	/**
	 * @return the contextListener
	 */
	public IContextListener getContextListener() {
		if(contextListener == null){
			throw new IllegalStateException("Context listener was not initialized");
		}
		return contextListener;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	@Override
	public void update(CdmDataChangeMap changeEvents) {
		// data changes can only be generate here at the moment
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operation.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	@Override
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		if(CdmStore.isActive()){
			viewer.setInput(getInput());
			viewer.refresh();
		}
		
		return true;
	}

	/**
	 * @return
	 */
	public abstract Collection getInput();
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#dispose()
	 */
	@Override
	public void dispose() {
		CdmStore.getContextManager().removeContextListener(getContextListener());
		super.dispose();
	}

	
	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public Viewer getViewer() {
		return viewer;
	}

	@Override
	public void createViewer(Composite parent) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void changed(Object object) {
		// do nothing
	}
	

	@Override
	public boolean onComplete() {
		return true;
	}

}
