// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.commands.Command;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.commands.ICommandService;

import eu.etaxonomy.taxeditor.model.MessagingUtils;

/**
 * Scans eu.etaxonomy.taxeditor.store.cdmViewer extension point.
 * @author pplitzner
 * @date Jul 7, 2015
 *
 */
public class CdmViewerUtil {

    /**
     * Returns a map of available commands to open the given input.
     * Keys are the command IDs and values are their string representations.
     *
     * @param input
     *            the object which should be handled by the available commands
     * @return a key-value map of available commands and their string
     *         representation
     */
    public static  Map<Command, String> getAvailableViewers(Object input){
        Map<Command, String> commandViewerNameMap = new HashMap<Command, String>();

        if(input!=null){
            IExtensionRegistry reg = Platform.getExtensionRegistry();
            IConfigurationElement[] extensions = reg
                    .getConfigurationElementsFor("eu.etaxonomy.taxeditor.store.cdmViewer"); //$NON-NLS-1$
            for (IConfigurationElement configElement : extensions) {
                if(configElement.getName().equals("viewCommandMapping")){ //$NON-NLS-1$
                    try {
                        String commandId = configElement.getAttribute("commandId"); //$NON-NLS-1$
                        String viewerName = configElement.getAttribute("viewerName"); //$NON-NLS-1$
                        Class<?> selectionClass = Class.forName(configElement.getAttribute("selection")); //$NON-NLS-1$
                        if(selectionClass.isAssignableFrom(input.getClass())){
                            ICommandService commandService = (ICommandService)PlatformUI.getWorkbench().getService(ICommandService.class);
                            Command command = commandService.getCommand(commandId);
                            //TODO: maybe pass the command directly instead of just the command id
                            if(command.isEnabled()){
                                commandViewerNameMap.put(command, viewerName);
                            }
                        }
                    } catch (ClassNotFoundException e) {
                        MessagingUtils.error(CdmViewerChooser.class, "Could not initalize selection class element of cdmViewer extension", e); //$NON-NLS-1$
                    }
                }
            }
        }
        return commandViewerNameMap;
    }

}
