// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.view.datasource.handler;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.progress.IProgressConstants;

import eu.etaxonomy.cdm.config.ICdmSource;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.database.update.CdmUpdater;
import eu.etaxonomy.taxeditor.model.CdmProgressMonitorAdapter;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;
import eu.etaxonomy.taxeditor.view.datasource.CdmDataSourceViewPart;
import eu.etaxonomy.taxeditor.view.datasource.CdmMetaDataAwareDataSourceContainer;

/**
 * <p>UpdateDataSourceHandler class.</p>
 *
 * @author n.hoffmann
 * @created Sep 22, 2010
 * @version 1.0
 */
public class UpdateDataSourceHandler extends AbstractDataSourceHandler {

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.datasource.handler.AbstractDataSourceHandler#specificExecute(org.eclipse.core.commands.ExecutionEvent)
	 */
	/** {@inheritDoc} */
	@Override
	public boolean specificExecute(ExecutionEvent event) {
		final CdmMetaDataAwareDataSourceContainer container = getSelectedDataSourceContainer(event);

		final Display display = Display.getCurrent();
		if(! MessageDialog.open(MessageDialog.CONFIRM, HandlerUtil.getActiveShell(event), "Update Datasource", "WARNING!\n\n" +
				"This will change your data base structure! Be sure you have an up-to-date backup of your data before running the update! \n\n " +
				"Are you sure you want to update the datasource?", SWT.NONE)){
			return false;
		}
		Job job = new Job("Updating datasource " + container.getCdmSource().getName()){

			@Override
			protected IStatus run(IProgressMonitor monitor) {
				CdmUpdater updater = new CdmUpdater();
				IStatus status = null;
				ICdmSource cdmSource = container.getCdmSource();
				//FIXME:Remoting we need another updater for ICdmRemoteSource
				if(cdmSource instanceof ICdmDataSource) {
				try{
					if(updater.updateToCurrentVersion((ICdmDataSource)cdmSource, CdmProgressMonitorAdapter.CreateMonitor(monitor))){
						display.asyncExec(new Runnable(){

							@Override
							public void run() {

								CdmDataSourceViewPart view = (CdmDataSourceViewPart) StoreUtil.getView(CdmDataSourceViewPart.ID, false);
								if(view != null){
								    container.getMetaDataFromDataSource();
									view.getViewer().update(new CdmMetaDataAwareDataSourceContainer[]{container}, null);
								}

							}
						});
						status = Status.OK_STATUS;
					}else{
						throw new RuntimeException("An error occurred during the update.");
					}
				}catch(Exception e){
					status = new Status(IStatus.ERROR, TaxeditorStorePlugin.PLUGIN_ID, e.getMessage(), e);
					MessagingUtils.errorDialog("Could not complete updater", updater, status.getMessage(), status.getPlugin(), e, true);
				}
				}

				return status;
			}

		};

		job.setPriority(Job.BUILD);
		job.setProperty(IProgressConstants.KEEP_PROPERTY, true);
		job.schedule();

		return false;
	}
}
