// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.store.operations;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.application.CdmChangeEvent.Action;
import eu.etaxonomy.cdm.api.service.ITestService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.api.service.dto.CdmEntityIdentifier;
import eu.etaxonomy.taxeditor.operation.RemotingCdmUpdateOperation;

/**
 * @author cmathew
 * @date 26 Jun 2015
 *
 */
public class RemotingTestUpdateOperation extends RemotingCdmUpdateOperation {

    private static String LABEL = "Test Operation";

    private UpdateResult result;
    private Exception exception;
    private CdmEntityIdentifier cei;


    /**
     * @param label
     * @param action
     * @param source
     * @param async
     */
    public RemotingTestUpdateOperation(UpdateResult result,
            Object source,
            boolean async) {
        super(LABEL, Action.Update, source, async);
        this.result = result;
    }

    public RemotingTestUpdateOperation(Exception exception,
            Object source,
            boolean async) {
        super(LABEL, Action.Update, source, async);
        this.exception = exception;
    }

    public RemotingTestUpdateOperation(CdmEntityIdentifier cei,
            Object source,
            boolean async) {
        super(LABEL, Action.Update, source, async);
        this.cei = cei;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmUpdateOperation#doUpdateExecute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
     */
    @Override
    protected UpdateResult doUpdateExecute(IProgressMonitor monitor, IAdaptable info) throws Exception {
        ITestService testService = CdmApplicationState.getTestService();
        if(result != null) {
            return testService.returnResult(result);
        } else if(exception != null) {
            return testService.throwException(exception);
        } else if(cei != null) {
            return testService.addChild(cei);
        }
        return null;
    }

}
