// $Id$
/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.occurrence.dna;

import eu.etaxonomy.cdm.model.common.DefinedTerm;
import eu.etaxonomy.cdm.model.common.TermType;
import eu.etaxonomy.cdm.model.molecular.Primer;
import eu.etaxonomy.cdm.model.molecular.SequenceString;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.supplemental.AnnotationSection;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @date 06.01.2014
 *
 */
public class PrimerGeneralDetailElement extends AbstractCdmDetailElement<Primer> {


    private TextWithLabelElement textPrimerName;
    // we are using a basic text field which is backed up by a Sequence object modelwise.
    // This is a bit of a overhead as we may just need the sequence string. This may change in future in both directions:
    // either extending this view to support the remaining fields of Sequence or by switching to the more basic SequenceString model element
    private TextWithLabelElement textPrimerSequence;
    private TermComboElement<DefinedTerm> comboMarker;
    private EntitySelectionElement<Reference> selectionReference;

    /**
     * @param formFactory
     * @param formElement
     */
    public PrimerGeneralDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#createControls
     * (eu.etaxonomy.taxeditor.forms.ICdmFormElement, java.lang.Object, int)
     */
    @Override
    protected void createControls(ICdmFormElement formElement, Primer entity, int style) {
        textPrimerName = formFactory.createTextWithLabelElement(formElement, "Primer Name", entity.getLabel(), style);
        String sequenceString = null;
        if(entity.getSequence()!=null){
            sequenceString = entity.getSequence().getString();
        }
        textPrimerSequence = formFactory.createTextWithLabelElement(formElement, "Primer seq. 5'->3'", sequenceString, style);

        comboMarker = formFactory.createDefinedTermComboElement(TermType.DnaMarker, formElement, "DNA Marker", entity.getDnaMarker(), style);

        selectionReference = formFactory
                .createSelectionElement(Reference.class,
                        getConversationHolder(), formElement, "Reference",
                        entity.getPublishedIn(),
                        EntitySelectionElement.ALL, style);

        AnnotationSection annotationSection = formFactory.createAnnotationSection(getConversationHolder(), formElement, style);
        annotationSection.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        annotationSection.setEntity(entity);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleEvent(java
     * .lang.Object)
     */
    @Override
    public void handleEvent(Object eventSource) {
        if(eventSource==textPrimerName){
            getEntity().setLabel(textPrimerName.getText());
        }
        else if(eventSource==comboMarker){
            getEntity().setDnaMarker(comboMarker.getSelection());
        }
        else if(eventSource==textPrimerSequence){
            SequenceString sequenceString = getEntity().getSequence();
            if(sequenceString==null){
                sequenceString = SequenceString.NewInstance("");
                getEntity().setSequence(sequenceString);
            }
            sequenceString.setString(textPrimerSequence.getText());
        }
        else if(eventSource==selectionReference){
            getEntity().setPublishedIn(selectionReference.getSelection());
        }
    }
}
